@@ -8,8 +8,9 @@ from .sc_config_mgr import SCConfigManager
 from .sc_csv_reader import CSVReader
 from .sc_date_helper import DateHelper
 from .sc_excel_reader import ExcelReader
+from .sc_json_encoder import JSONEncoder
 from .sc_logging import SCLogger
 from .sc_shelly_control import ShellyControl
 from .webhook_server import _ShellyWebhookHandler
 
-__all__ = ["CSVReader", "DateHelper", "ExcelReader", "SCCommon", "SCConfigManager", "SCLogger", "ShellyControl", "_ShellyWebhookHandler"]
+__all__ = ["CSVReader", "DateHelper", "ExcelReader", "JSONEncoder", "SCCommon", "SCConfigManager", "SCLogger", "ShellyControl", "_ShellyWebhookHandler"]
